/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "current.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define CURRENT_DUMMY 0

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void current_generic_transfer ( current_t *ctx, spi_master_transfer_data_t *block );

static void current_calc_delay ();

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void current_cfg_setup ( current_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

CURRENT_RETVAL current_init ( current_t *ctx, current_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return CURRENT_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, CURRENT_DUMMY );

    return CURRENT_OK;

}

uint16_t current_data_read ( current_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t rx_buf[ 2 ];
    uint16_t result;

    block.tx_length = 0;
    block.rx_buffer = rx_buf;
    block.rx_length = 2;

    current_generic_transfer( ctx, &block );
    
    result = rx_buf[ 0 ];
    result <<= 8;
    result |= rx_buf[ 1 ];
    return result;
}

uint16_t current_get_raw_data ( current_t *ctx )
{
    uint16_t result;

    result = current_data_read(ctx );

    result >>= 1;
    result &= CURRENT_FILTER_USEFULL_DATA;

    return result;
}

float current_avg_rata ( current_t *ctx )
{
    uint16_t temp;
    float sum;
    uint8_t n_cnt;
    
    sum = 0;

    for ( n_cnt = 0; n_cnt < CURRENT_SAMPLE_NUM; n_cnt++ )
    {
        temp = current_get_raw_data( ctx );
        current_calc_delay();
        sum += temp;
    }

    sum /= CURRENT_SAMPLE_NUM;

    return sum;
}

float current_get_current_data ( current_t *ctx, float r_shunt )
{
    float result;
    float avg_data;

    avg_data = current_get_raw_data( ctx );

    if( avg_data < RFMETER_DEF_LIMIT_HIGH && avg_data > RFMETER_DEF_LIMIT_LOW )
    {
        result = ( float ) avg_data;
        result *= CURRENT_CAL_COEF;
        result /= r_shunt;
    }
    else
    {
        result = CURRENT_OUT_OF_RANGE;
    }

    return result;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void current_generic_transfer ( current_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

static void current_calc_delay ()
{
    Delay_10ms();
}

// ------------------------------------------------------------------------- END

